unit MainDlg;

interface

{$INCLUDE PictureTemplate.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, CamRemote, CamDefines, ExtCtrls, ComCtrls, DlgSelectCamera,
  OleCtrls, CamRemoteActiveXXControl1_TLB;

type
  TFormMain = class(TForm)
    ButtonConnect: TButton;
    ListBox: TListBox;
    RichEdit: TRichEdit;
    ButtonDisconnect: TButton;
    EditPCDir: TEdit;
    LabelPCDir: TLabel;
    ImageThumbnail: TImage;
    ButtonTransfer: TButton;
    ButtonDelete: TButton;
    ProgressBar: TProgressBar;
    CheckBoxThumbnails: TCheckBox;
    CamRemoteActiveX: TCamRemoteActiveXX;
    procedure ButtonConnectClick(Sender: TObject);
    procedure ButtonDisconnectClick(Sender: TObject);
    procedure ListBoxClick(Sender: TObject);
    procedure ButtonTransferClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ButtonDeleteClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CamRemoteActiveXGetPictureEvent(ASender: TObject;
      PercentageDone: Integer);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    mCameraEnumerated : boolean;             //Is true when camera enumeration has been performed
    procedure UpdateListBox;
    function  Versiontext : string;
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

{$R *.dfm}

{$IFDEF DEXIF}
uses dEXIF, dIPTC;
{$ENDIF}

procedure TFormMain.ButtonConnectClick(Sender: TObject);
var i : integer;
begin
  CamRemoteActiveX.OpenCameraEnumeration;
  mCameraEnumerated := true;
  if ((CamRemoteActiveX.OpenCameraEnumeration_GetPowerShotNr = 0) and
      (CamRemoteActiveX.OpenCameraEnumeration_GetEOSNr = 0)) then
  begin
    raise ECamException.Create('No camera is connected to the computer');
  end;
  FormSelectCamera.ModalResult := mrOK;
  FormSelectCamera.ComboBoxCameras.Items.Clear;
  //Are there any PowerShot cameras connected?
  if (CamRemoteActiveX.OpenCameraEnumeration_GetPowerShotNr > 0) then
  begin
    if (CamRemoteActiveX.OpenCameraEnumeration_GetPowerShotNr > 1) then
    begin
      //Select which camera to connect to
      for i := 0 to (CamRemoteActiveX.OpenCameraEnumeration_GetPowerShotNr - 1) do
      begin
        FormSelectCamera.ComboBoxCameras.Items.Add(CamRemoteActiveX.OpenCameraEnumeration_GetModel(CamModPowerShot, i));
      end;
      FormSelectCamera.ComboBoxCameras.ItemIndex := 0;
      if (FormSelectCamera.ShowModal = mrOK) then
      begin
        CamRemoteActiveX.Connect(CamModPowerShot,
                                 FormSelectCamera.ComboBoxCameras.ItemIndex);
      end;
    end else begin
      //Only one PowerShot camera connected. Select that camera
      CamRemoteActiveX.Connect(CamModPowerShot, 0);
    end;
  end else begin
    //No PowerShot cameras. Connect to EOS camera
    if (CamRemoteActiveX.OpenCameraEnumeration_GetEOSNr > 1) then
    begin
      //Select which camera to connect to
      for i := 0 to (CamRemoteActiveX.OpenCameraEnumeration_GetEOSNr - 1) do
      begin
        FormSelectCamera.ComboBoxCameras.Items.Add(CamRemoteActiveX.OpenCameraEnumeration_GetModel(CamModEOS, i));
      end;
      FormSelectCamera.ComboBoxCameras.ItemIndex := 0;
      if (FormSelectCamera.ShowModal = mrOK) then
      begin
        CamRemoteActiveX.Connect(CamModEOS,
                                 FormSelectCamera.ComboBoxCameras.ItemIndex);
      end;
    end else begin
      //Only one EOS camera connected. Select that camera
        CamRemoteActiveX.Connect(CamModEOS, 0);
    end;
  end;
  if (FormSelectCamera.ModalResult <> mrOK) then
  begin
    Exit;
  end;
  //List all pictures on the camera
  CamRemoteActiveX.OpenCameraCollection(ExtractFileDir(ParamStr(0)),
                                        CheckBoxThumbnails.Checked);
  UpdateListBox;
end;

procedure TFormMain.UpdateListBox;
var i : integer;
begin
  //Update the list of pictures on the form
  ListBox.Items.Clear;
  for i := 0 to CamRemoteActiveX.OpenCameraCollection_GetNrOfPictures - 1 do
  begin
    ListBox.Items.Add(CamRemoteActiveX.OpenCameraCollection_NameOfImage(i))
  end;
end;

procedure TFormMain.ButtonDisconnectClick(Sender: TObject);
begin
  //Close the picture collection and delete the objects in mImageList
  CamRemoteActiveX.CloseCameraCollection;
  CamRemoteActiveX.Disconnect;
end;

procedure TFormMain.ListBoxClick(Sender: TObject);
var thumb_file_name : string;
begin
  //Print EXIF data for RAW-pictures to a RichEdit VCL.
  RichEdit.Lines.Clear;
  if ((ListBox.ItemIndex <> -1) and (CheckBoxThumbnails.Checked)) then
  begin
    thumb_file_name := GetEnvironmentVariable('Temp') + '\Thumb.jpg';
    CamRemoteActiveX.OpenCameraCollection_GetThumbnail(ListBox.ItemIndex, thumb_file_name);
    //Update thumbnail on the form
    ImageThumbnail.Picture.LoadFromFile(thumb_file_name);
  end;
end;

procedure TFormMain.ButtonTransferClick(Sender: TObject);
begin
  if (ListBox.ItemIndex <> -1) then
  begin
    //Get JPEG in RAW picture.
    if (CamRemoteActiveX.OpenCameraCollection_RAWJpegExists(ListBox.ItemIndex)) then
    begin
      CamRemoteActiveX.GetPicture(ListBox.ItemIndex,
                                  true,
                                  ChangeFileExt(EditPCDir.Text + '\' + ListBox.Items[ListBox.ItemIndex],
                                                '.jpg'));
    end;
    CamRemoteActiveX.GetPicture(ListBox.ItemIndex,
                                false,
                                EditPCDir.Text + '\' + ListBox.Items[ListBox.ItemIndex]);
  end;
  ProgressBar.Position := 0;
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  EditPCDir.Text       := ExtractFileDir(ParamStr(0));
  FormMain.Caption     := FormMain.Caption + ' version ' + Versiontext;
  ProgressBar.Position := 0;
  mCameraEnumerated    := false;
end;

procedure TFormMain.ButtonDeleteClick(Sender: TObject);
begin
  if (ListBox.ItemIndex <> -1) then
  begin
    CamRemoteActiveX.DeletePicture(ListBox.ItemIndex);
    //Update the list of pictures, since mImageList has been modified
    UpdateListBox;
  end else begin
    MessageDlg('No picture selected to delete', mtInformation, [mbOK], 0)
  end;
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if (mCameraEnumerated) then
  begin
    CamRemoteActiveX.CloseCameraEnumeration;
    mCameraEnumerated := false;
  end;
end;

//Gets application version text
function TFormMain.Versiontext : string;
var
  size  : DWord;
  size2 : DWord;
  pt    : Pointer;
  pt2   : Pointer;
begin
  //Get the version of application
  result := '';
  size   := GetFileVersionInfoSize(PChar(ParamStr(0)), size2);
  if (size > 0) then
  begin
    GetMem(pt, size);
    try
       GetFileVersionInfo(PChar(ParamStr(0)), 0, size, pt);
       // show the fixed information
       VerQueryValue(pt, '\', pt2, size2);
       with TVSFixedFileInfo(pt2^) do
       begin
         result := result + IntToStr(HiWord(dwFileVersionMS)) + '.';
         result := result + IntToStr(LoWord(dwFileVersionMS)) + '.';
         result := result + IntToStr(HiWord(dwFileVersionLS));
         result := result + ' Build:' + IntToStr(LoWord(dwFileVersionLS));
       end;
    finally
      FreeMem(pt);
    end;
  end;
end;

procedure TFormMain.CamRemoteActiveXGetPictureEvent(ASender: TObject;
  PercentageDone: Integer);
begin
  ProgressBar.Position := PercentageDone;
end;

procedure TFormMain.FormShow(Sender: TObject);
begin
  CamRemoteActiveX.Visible := false;
end;

end.
